## TODO: make module this package conform to a single style for input/output

def read_cadnano(json_file, sequence=None, fill_sequence='T', **model_parameters):
    from .cadnano_segments import read_json_file
    from .cadnano_segments import read_model as model_from_cadnano_json

    data = read_json_file(json_file)
    return model_from_cadnano_json(data, sequence, fill_sequence, **model_parameters)

def read_cadnano_input(json_file,sequence=None,**model_parameters):
    from .segmentmodel_from_cadnano import mrdna_model_from_cadnano
    from .segmentmodel_from_lists import model_from_basepair_stack_3prime
    

    return mrdna_model_from_cadnano(json_file,seq=sequence,**model_parameters)

def read_vhelix(maya_file, **model_parameters):
    from .polygon_mesh import parse_maya_file, convert_maya_bases_to_segment_model

    maya_bases = parse_maya_file(maya_file)
    model = convert_maya_bases_to_segment_model( maya_bases, **model_parameters )
    return model

def read_list(infile,**model_parameters):
    import numpy as np
    try:
        data = np.loadtxt(infile)
    except:
        data = np.loadtxt(infile,skiprows=1) # strip header
    coords = data[:,:3]*10
    bp = data[:,3]
    stack = data[:,4]
    three_prime = data[:,5]

    from .segmentmodel_from_lists import model_from_basepair_stack_3prime
    return model_from_basepair_stack_3prime(coords, bp, stack, three_prime)


def read_atomic_pdb(pdb_file, **model_parameters):
    from .segmentmodel_from_pdb import SegmentModelFromPdb
    return SegmentModelFromPdb(pdb_file)

def read_oxdna(coordinate_file, topology_file, idealized_coordinate_file=None, **model_parameters):
    """ Idealized coordinate file: coordinates for detecting stacks and base pairs, defaults to coordinate_file """ 
    from .segmentmodel_from_oxdna import mrdna_model_from_oxdna
    return mrdna_model_from_oxdna(coordinate_file, topology_file, idealized_coordinate_file, **model_parameters)
