# -*- coding: utf-8 -*-
import pdb
import numpy as np
import os,sys
from glob import glob
import re
import pandas as pd
pd.options.mode.chained_assignment = None  # default='warn'
from .segmentmodel_from_lists import model_from_basepair_stack_3prime

from ..arbdmodel.coords import readArbdCoords, readAvgArbdCoords, rotationAboutAxis
from ..segmentmodel import SegmentModel, SingleStrandedSegment, DoubleStrandedSegment
from ..model.dna_sequence import m13 as m13seq
import json
import re
import cadnano
from cadnano.document import Document


## Only testing on cadnano2.5
## TODO: separate SegmentModel from ArbdModel so multiple parts can be combined
## TODO: catch circular strands in "get_5prime" cadnano calls
## TODO: handle special motifs
##   - doubly-nicked helices
##   - helices that should be stacked across an empty region (crossovers from and end in the helix to another end in the helix)
##   - circular constructs
def get_lattice(part):
    lattice_type = None
    _gt = part.getGridType()
    try:
        lattice_type = _gt.name.lower()
    except:
        if _gt == 1:
            lattice_type = 'square'
        elif _gt == 2:
            lattice_type = 'honeycomb'
        else:
            print("WARNING: unable to determine cadnano part lattice type")
    return lattice_type


def read_json_file(filename):

    try:
        with open(filename) as ch:
            json_data = json.load(ch)
    except:
        with open(filename) as ch:
            content = ""
            for l in ch:
                l = re.sub(r"'", r'"', l)
                # https://stackoverflow.com/questions/4033633/handling-lazy-json-in-python-expecting-property-name
                # l = re.sub(r"{\s*(\w)", r'{"\1', l)
                # l = re.sub(r",\s*(\w)", r',"\1', l)
                # l = re.sub(r"(\w):", r'\1":', l)
                content += l+"\n"
            json_data = json.loads(content)

    try:
        doc = Document()
        cadnano.fileio.v3decode.decode(doc, json_data)
        decoder = 3
    except:
        doc = Document()
        cadnano.fileio.v2decode.decode(doc, json_data)
        decoder = 2

    parts = [p for p in doc.getParts()]
    if len(parts) != 1:
        raise Exception("Only documents containing a single cadnano part are implemented at this time.")
    part = parts[0]

    if decoder == 2:
        """ It seems cadnano2.5 (as of ce6ff019) does not set the EulerZ for square lattice structures correctly, doing so here """
        l = get_lattice(part)
        if l == 'square':
            for id_num in part.getIdNums():
                if part.vh_properties.loc[id_num,'eulerZ'] == 0:
                    part.vh_properties.loc[id_num,'eulerZ'] = 360*(6/10.5)
    df=pd.DataFrame(json_data["vstrands"])
    n_df=df.set_index("num")
    return part,n_df

def get_helix_angle(part, helix_id, indices):
    """ Get "start_orientation" for helix """
        # import ipdb
        # ipdb.set_trace()

    """ FROM CADNANO2.5
    + angle is CCW
    - angle is CW
    Right handed DNA rotates clockwise from 5' to 3'
    we use the convention the 5' end starts at 0 degrees
    and it's pair is minor_groove_angle degrees away
    direction, hence the minus signs.  eulerZ
    """

    hp, bpr, tpr, eulerZ, mgroove = part.vh_properties.loc[helix_id,
                                                                    ['helical_pitch',
                                                                     'bases_per_repeat',
                                                                     'turns_per_repeat',
                                                                     'eulerZ',
                                                                     'minor_groove_angle']]
    twist_per_base = tpr*360./bpr
        # angle = eulerZ - twist_per_base*indices + 0.5*mgroove + 180
    angle = eulerZ + twist_per_base*indices - 0.5*mgroove
    return rotationAboutAxis(np.array((0,0,1)),angle)

def nttype(scafs):
    def judge(i):
        if i ==[-1,-1,-1,-1]:
            return 0
        else: return 1
    n=np.array([judge(i) for i in scafs])
    return n

            
def gen_prop_table(json_file):
    part,vslist=read_json_file(json_file)
    props = part.getModelProperties().copy()
    try:
        if props.get('point_type') == PointType.ARBITRARY:
            # TODO add code to encode Parts with ARBITRARY point configurations
            raise NotImplementedError("Not implemented")
    except:
        try:
            vh_props, origins = part.helixPropertiesAndOrigins()
        except:
            origins = {hid:part.getVirtualHelixOrigin(hid)[:2] for hid in part.getidNums()}
    scaf_id=[nttype(vslist['scaf'][i]) for i in vslist.index]
    stap_id=[nttype(vslist['stap'][i]) for i in vslist.index]
    cad_bps=part.getIndices(0)
    vslist["scafnt"]=np.sum(np.array(scaf_id),axis=1)
    vslist["stapnt"]=np.sum(np.array(stap_id),axis=1)
    totnt=np.sum(vslist["scafnt"])+np.sum(vslist["stapnt"])
    is_scaf=np.zeros(totnt,dtype=bool)
    is_scaf[0:np.sum(vslist["scafnt"])]=1
    nt_prop=pd.DataFrame(index=range(totnt),columns=["vh","zid","is_scaf","r","bp","stack","threeprime","seq","orientation"])
    nt_prop["is_scaf"]=is_scaf
    tot_id=scaf_id+stap_id
    vhi,zidi=np.where(np.array(scaf_id)==1)
    vhj,zidj=np.where(np.array(stap_id)==1)
    vhi=vslist.index[vhi]
    vhj=vslist.index[vhj]
    nt_prop["vh"]=list(vhi)+list(vhj)
    nt_prop["zid"]=list(zidi)+list(zidj)
    vhzid=list(zip(nt_prop["vh"],nt_prop["zid"]))
    nt_prop["r"]=[part.getCoordinate(helix_id, indices) for helix_id, indices in zip(nt_prop["vh"],nt_prop["zid"])]
    nt_prop["orientation"]=[get_helix_angle(part, helix_id, indices) for helix_id,indices in zip(nt_prop["vh"],nt_prop["zid"])]
    nt_prop=nt_prop.fillna(-1)
    for i in range(int(len(vhzid)/2)):
        try:
            bp1,bp2=(i,1+i+vhzid[i+1:].index(vhzid[i]))
            nt_prop["bp"][bp1]=bp2
            nt_prop["bp"][bp2]=bp1
        except:
            pass
    tprime_list=-np.ones(len(nt_prop.index),dtype=int)
    index2=list(zip(vhzid,nt_prop["is_scaf"]))
    for i in range(len(nt_prop.index)):
        ((m,n),p)=list(zip(vhzid,nt_prop["is_scaf"]))[i]
        if p==True:
            k,l=(vslist["scaf"][m])[n][2:]
            if k!=-1 and l!=-1:
                n=index2.index(((k,l),True))
                tprime_list[i]=int(n)
            
        else:
            k,l=(vslist["stap"][m])[n][2:]
            if k!=-1 and l!=-1:
                n=index2.index(((k,l),False))
                tprime_list[i]=int(n)
    nt_prop["threeprime"]=tprime_list
    (n,)=np.where(nt_prop["threeprime"]==-1)
    stackid=nt_prop["bp"][[list(nt_prop["threeprime"]).index(i) for i in n]]
    nt_prop["stack"][stackid.index[np.where(np.array(stackid)!=-1)]]=nt_prop["threeprime"][stackid.index[np.where(np.array(stackid)!=-1)]]
    ## Todo: sequence    
    

    return nt_prop

def mrdna_model_from_cadnano(json_file,seq=None,**model_parameters):
    nt_prop=gen_prop_table(json_file)
    if seq is None:
        if nt_prop["seq"][0]==-1:
            seq=None
        else:
            seq=nt_prop["seq"]
    
    r=np.array(list(nt_prop['r']))
    bp=np.array(list(nt_prop['bp']))
    three_prime=np.array((list(nt_prop["threeprime"])))
    stack=np.array((list(nt_prop["stack"])))
    orientation=np.array(list(nt_prop["orientation"]))
    model = model_from_basepair_stack_3prime( r, bp, stack, three_prime, seq, orientation=None, **model_parameters )
    return model
